﻿namespace iFileProxy
{
    using Serilog;
    using Serilog.Events;
    using System.Net;

    public static class SerilogConfig
    {
        public static void CreateLogger()
        {
            var baseLogPath = Path.Combine(AppContext.BaseDirectory, "logs");
            var appName = AppDomain.CurrentDomain.FriendlyName;
            
            // 确保日志目录存在
            if (!Directory.Exists(baseLogPath))
            {
                Directory.CreateDirectory(baseLogPath);
            }

            Log.Logger = new LoggerConfiguration()
#if RELEASE
                .MinimumLevel.Information()
#else
                .MinimumLevel.Debug()
#endif
                .MinimumLevel.Override("Microsoft", LogEventLevel.Information)
                .MinimumLevel.Override("Microsoft.AspNetCore", LogEventLevel.Warning)
                .Enrich.FromLogContext()
                // 控制台输出所有日志
                .WriteTo.Console(
                    outputTemplate: "{Timestamp:HH:mm:ss.fff} [{Level:u3}] [{SourceContext}] {ClientIp} {Message:lj} {contentType}{NewLine} {Exception}")
                // 错误和致命错误写入单独的文件
                .WriteTo.Logger(lc => lc
                    .Filter.ByIncludingOnly(evt => evt.Level >= LogEventLevel.Error)
                    .WriteTo.File(
                        Path.Combine(baseLogPath, $"{appName}.error.log"),
                        outputTemplate: "{Timestamp:yyyy-MM-dd HH:mm:ss.fff zzz} [{Level:u3}] [{SourceContext}] {ClientIp} {Message:lj} {contentType} {queryString}{NewLine}{Exception}",
                        rollingInterval: RollingInterval.Day,
                        fileSizeLimitBytes: 1073741824)) // 1GB
                // 警告日志写入单独的文件
                .WriteTo.Logger(lc => lc
                    .Filter.ByIncludingOnly(evt => evt.Level == LogEventLevel.Warning)
                    .WriteTo.File(
                        Path.Combine(baseLogPath, $"{appName}.warning.log"),
                        outputTemplate: "{Timestamp:yyyy-MM-dd HH:mm:ss.fff zzz} [{Level:u3}] [{SourceContext}] {ClientIp} {Message:lj} {contentType} {queryString}{NewLine}{Exception}",
                        rollingInterval: RollingInterval.Day,
                        fileSizeLimitBytes: 1073741824))
                // 信息日志写入单独的文件
                .WriteTo.Logger(lc => lc
                    .Filter.ByIncludingOnly(evt => evt.Level == LogEventLevel.Information)
                    .WriteTo.File(
                        Path.Combine(baseLogPath, $"{appName}.info.log"),
                        outputTemplate: "{Timestamp:yyyy-MM-dd HH:mm:ss.fff zzz} [{Level:u3}] [{SourceContext}] {ClientIp} {Message:lj} {contentType} {queryString}{NewLine}{Exception}",
                        rollingInterval: RollingInterval.Day,
                        fileSizeLimitBytes: 1073741824))
                // 调试日志写入单独的文件
                .WriteTo.Logger(lc => lc
                    .Filter.ByIncludingOnly(evt => evt.Level == LogEventLevel.Debug)
                    .WriteTo.File(
                        Path.Combine(baseLogPath, $"{appName}.debug.log"),
                        outputTemplate: "{Timestamp:yyyy-MM-dd HH:mm:ss.fff zzz} [{Level:u3}] [{SourceContext}] {ClientIp} {Message:lj} {contentType} {queryString}{NewLine}{Exception}",
                        rollingInterval: RollingInterval.Day,
                        fileSizeLimitBytes: 1073741824))
                .Enrich.WithProperty("node_ip", GetIpAddress())
                .CreateLogger();
        }

        public static void RefreshLogger()
        {
            if (Log.Logger != null)
            {
                Log.CloseAndFlush();
            }
            CreateLogger();
        }

        private static string GetIpAddress()
        {
            string ipAddress = "127.0.0.1";
            IPAddress[] ips = Dns.GetHostAddresses(Dns.GetHostName());
            foreach (IPAddress ip in ips)
            {
                if (ip.AddressFamily.ToString().ToLower().Equals("internetwork"))
                {
                    ipAddress = ip.ToString();
                    return ipAddress;
                }
            }

            return ipAddress;
        }
    }
}
